/*
** Module   : CopySpec.java
** Abstract : defines a file copy operation
**
** Copyright (c) 2007-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- -------------------Description--------------------
** 001 GES 20070716   @34527 Created initial version. Defines a file copy
                             operation (source, target and type of copy).
** 002 GES 20130206          Moved class to the pattern package.
*/                                                            

package com.goldencode.p2j.pattern;

/**
 * Defines a file copy operation (source, target and type of copy).
 * <p>
 * The following copy operation types are defined:
 * <p>
 * <pre>
 * Type                  Description
 * -------------------   ----------------------------------------
 * Type.BINARY           A byte for byte copy.
 * Type.TXT_TO_HTML      Each line of the input text file will be
 *                       copied to the output file as a line numbered
 *                       named anchor inside HTML preformatted tags.
 * </pre>
 */
public class CopySpec
{
   /** Operation type constants. */
   public enum Type { BINARY, TXT_TO_HTML };
   
   /** Source filename. */
   private String source = null;
   
   /** Target filename. */
   private String target = null;
   
   /** Operation. */
   private Type op = Type.BINARY;
   
   /**
    * Create an instance with the given filenames. The copy operation type
    * will default to <code>Type.BINARY</code>.
    *
    * @param    source
    *           Source filename.
    * @param    target
    *           Target filename.
    */
   public CopySpec(String source, String target)
   {
      this.source = source;
      this.target = target;
   }
   
   /**
    * Create an instance with the given filenames.
    *
    * @param    source
    *           Source filename.
    * @param    target
    *           Target filename.
    * @param    op
    *           Copy operation to use.
    */
   public CopySpec(String source, String target, Type op)
   {
      this(source, target);
      this.op = op;
   }
   
   /**
    * Access the source filename.
    *
    * @return   Source.
    */
   public String getSource()
   {
      return source;
   }
   
   /**
    * Set the source filename.
    *
    * @param    source
    *           New source filename.
    */
   public void setSource(String source)
   {
      this.source = source;
   }
   
   /**
    * Access the target filename.
    *
    * @return   Target.
    */
   public String getTarget()
   {
      return target;
   }
   
   /**
    * Set the target filename.
    *
    * @param    target
    *           New target filename.
    */
   public void setTarget(String target)
   {
      this.target = target;
   }
   
   /**
    * Access the copy operation.
    *
    * @return   Copy operation.
    */
   public Type getOperation()
   {
      return op;
   }
   
   /**
    * Set the copy operation type.
    *
    * @param    op
    *           New copy operation type.
    */
   public void setOperation(Type op)
   {
      this.op = op;
   }
}
